/*
====================================================================

Program:             APMMONIT.CPP

Description:         Advanced Power Management monitor

Date created:        00/07/27

Date last updated:   00/07/27

                     01/11/15 - FC - Enhanced for Windows

                     01/11/19 - FC - Optimized to display only the changes in the power management.

Notes:               This program will verify battery information every second
                     and display the results on the screen.  This application is
                     used mainly to verify the working state of the battery and
                     whether the battery suffers from any sudden power loss.


(C) Copyright 2000, 2001 Circuit Design Corporation
====================================================================
*/



#include <cdc.hpp>



int main(void)
{

 int rc;

 SYSTEM_POWER_STATUS sps,
                     old_sps;

 char current_date[9],
      current_time[9];

 BOOL initial_start = TRUE;


   memset(&sps    , 0, sizeof(SYSTEM_POWER_STATUS));
   memset(&old_sps, 0, sizeof(SYSTEM_POWER_STATUS));

   setbuf(stdout, 0);

   printf("\nAPM Battery Monitor\n");
   printf("(C) Copyright 2001 Circuit Design Corporation\n\n");

   printf("Beginning monitoring...\n\n");

   while (TRUE) {

      Sleep(1000);

      _strdate(current_date);
      _strtime(current_time);

      rc = GetSystemPowerStatus(&sps);

      if (rc == 0) {
         printf("APM driver access failed!  Error code = %ld\n", GetLastError());
         return ERROR;
      }

      if (sps.ACLineStatus == 255) {
         printf("APM driver is currently DISABLED!\n");
         return ERROR;
      }

      if (memcmp(&sps, &old_sps, sizeof(SYSTEM_POWER_STATUS)) == 0) continue;

      if (!initial_start && abs((int)(sps.BatteryLifePercent) - (int)(old_sps.BatteryLifePercent)) > 1) {
         printf("*** Battery life has changed more than one percent ***\n");
      }

      initial_start = FALSE;

      printf("%s %s  Power source: ", current_date, current_time);

      switch (sps.ACLineStatus) {

         case 0x00:
            printf("Battery  ");
            break;

         case 0x01:
            printf("AC       ");
            break;

         default:
            printf("Unknown  ");
            break;

      }

      printf("Status: ");

      if (sps.BatteryFlag == 0xff) {

         printf("Unknown\n");

      } else {

         if ((sps.BatteryFlag & 0x01) == 0x01) printf("High, "      );
         if ((sps.BatteryFlag & 0x02) == 0x02) printf("Low, "       );
         if ((sps.BatteryFlag & 0x04) == 0x04) printf("Critical, "  );
         if ((sps.BatteryFlag & 0x08) == 0x08) printf("Charging, "  );
         if ((sps.BatteryFlag & 0x80) == 0x80) printf("No battery, ");

         switch (sps.BatteryLifePercent) {

            case 0xff:
               printf("??%%\n");
               break;

            default:
               printf("%d%%\n", sps.BatteryLifePercent);
               break;

         }

      }

      if (sps.BatteryLifeTime     != 0xffffffff) printf("Battery life remaining     : %ld seconds.\n", sps.BatteryLifeTime    );
      if (sps.BatteryFullLifeTime != 0xffffffff) printf("Full battery life available: %ld seconds.\n", sps.BatteryFullLifeTime);

      memcpy(&old_sps, &sps, sizeof(SYSTEM_POWER_STATUS));

   }


   return OK;

}
